import 'dart:math';

import 'package:axlehire_flutter_model/axlehire_model.dart';
import 'package:flutter_polyline_points/flutter_polyline_points.dart';
import 'package:json_annotation/json_annotation.dart';

import 'geolocation.dart';
part 'assignment_extra.g.dart';

/// An annotation for the code generator to know that this class needs the
/// JSON serialization logic to be generated.
@JsonSerializable()

class AssignmentExtra {
  AssignmentExtra(this.type, this.id, this.assignmentId, this.dropoffStatus, this.path, this.fullPath);

  @JsonKey(name: "assignment_id")
  final int assignmentId;
  final String id;
  final String type;
  @JsonKey(name: "dropoff_status")
  final String dropoffStatus; // summary of dropoff status S: succeeded, F: failed, P: Pending
  final String path;
  @JsonKey(name: "full_path")
  final String fullPath;

  static PolylinePoints polylinePoints = PolylinePoints();

  List<Geolocation> get geolocations {
    if (fullPath == null || fullPath.isEmpty) return [];
    List<PointLatLng> points = polylinePoints.decodePolyline(fullPath);
    return points.map((p) => Geolocation(p.latitude, p.longitude, null)).toList();
  }

  List<Geolocation> get dropoffLocations {
    if (dropoffStatus == null || dropoffStatus.isEmpty) return [];

    List<String> stopTypes = dropoffStatus.split('|').map((s) => s[0]).toList();
    List<Geolocation> locs = this.geolocations;

    List<Geolocation> _dropoffCoordinates = [];

    for (var i = 0; i < stopTypes.length; i++) {
      if (stopTypes[i] == 'D') {
        _dropoffCoordinates.add(locs[i]);
      }
    }
    return _dropoffCoordinates;
  }

  List<Geolocation> get pickupLocations {
    if (dropoffStatus == null || dropoffStatus.isEmpty) return [];

    List<String> stopTypes = dropoffStatus.split('|').map((s) => s[0]).toList();
    List<Geolocation> locs = this.geolocations;

    List<Geolocation> _pickupCoordinates = [];

    for (var i = 0; i < stopTypes.length; i++) {
      if (stopTypes[i] == 'P') {
        _pickupCoordinates.add(locs[i]);
      }
    }
    return _pickupCoordinates;
  }

  Tuple<Geolocation, Geolocation, Geolocation> get dropoffBoundary {
    List<Geolocation> _dropoffCoordinates = this.dropoffLocations;
    var lats = _dropoffCoordinates.map((d) => d.latitude).toList();
    var lngs = _dropoffCoordinates.map((d) => d.longitude).toList();
    var maxLat = lats.reduce(max);
    var minLat = lats.reduce(min);
    var maxLng = lngs.reduce(max);
    var minLng = lngs.reduce(min);
    var centLat = lats.reduce((a,b) => a + b) / lats.length;
    var centLng = lngs.reduce((a,b) => a + b) / lats.length;

    return Tuple(Geolocation(minLat, minLng, null), Geolocation(maxLat, maxLng, null), Geolocation(centLat, centLng, null));
  }

  factory AssignmentExtra.fromJson(Map<String, dynamic> json) => _$AssignmentExtraFromJson(json);
  Map<String, dynamic> toJson() => _$AssignmentExtraToJson(this);

}