import 'dart:convert';
import 'package:amazon_cognito_identity_dart/sig_v4.dart';

import '../axlehire_model.dart';

class AWSS3Policy {
  static const String DEFAULT_REGION = "us-west-2";

  String expiration;
  String region;
  String bucket;
  String key;
  String credential;
  String securityToken;
  String tokenType;
  String datetime;
  int maxFileSize;

  AWSS3Policy(this.key, this.bucket, this.datetime, this.expiration, this.credential,
    this.maxFileSize,
    {this.securityToken, this.region = DEFAULT_REGION, this.tokenType = AWSSecurityToken.TYPE_FEDERATION});

  factory AWSS3Policy.fromS3PresignedPost(
    String key,
    String bucket,
    String accessKeyId,
    int expiryMinutes,
    int maxFileSize,
    {
      String securityToken,
      String region = DEFAULT_REGION,
    }) {
    final datetime = SigV4.generateDatetime();
    final expiration = (DateTime.now())
      .add(Duration(minutes: expiryMinutes))
      .toUtc()
      .toString()
      .split(' ')
      .join('T');
    final cred = '$accessKeyId/${SigV4.buildCredentialScope(datetime, region, 's3')}';
    return AWSS3Policy(
      key, bucket, datetime, expiration, cred, maxFileSize,
      securityToken: securityToken, region: region);
  }

  String encode() {
    final bytes = utf8.encode(toString());
    return base64.encode(bytes);
  }

  @override
  String toString() {
    return this.securityToken != null ?
    '''
      { "expiration": "${this.expiration}",
        "conditions": [
          {"bucket": "${this.bucket}"},
          ["starts-with", "\$key", "${this.key}"],
          {"acl": "bucket-owner-full-control"},
          ["content-length-range", 1, ${this.maxFileSize}],
          {"x-amz-credential": "${this.credential}"},
          {"x-amz-algorithm": "AWS4-HMAC-SHA256"},
          {"x-amz-date": "${this.datetime}" },
          {"x-amz-security-token": "${this.securityToken}" }
        ]
      }''' :
    '''
      { "expiration": "${this.expiration}",
        "conditions": [
          {"bucket": "${this.bucket}"},
          ["starts-with", "\$key", "${this.key}"],
          {"acl": "bucket-owner-full-control"},
          ["content-length-range", 1, ${this.maxFileSize}],
          {"x-amz-credential": "${this.credential}"},
          {"x-amz-algorithm": "AWS4-HMAC-SHA256"},
          {"x-amz-date": "${this.datetime}" }
        ]
      }''';
  }
}