import 'package:axlehire_flutter_model/axlehire_model.dart';
import 'package:axlehire_flutter_model/src/assignment.dart';
import 'package:axlehire_flutter_model/src/assignment_extra.dart';
import 'package:axlehire_flutter_model/src/decor_stop.dart';
import 'package:axlehire_flutter_model/src/geocode_address.dart';
import 'package:axlehire_flutter_model/src/mixin/metadata.dart';
import 'package:axlehire_flutter_model/src/task.dart';
import 'package:axlehire_flutter_model/src/task_type.dart';
import 'package:json_annotation/json_annotation.dart';
part 'decor_assignment.g.dart';

@JsonSerializable()

class DecorAssignment with MetaDataMixin {
  DecorAssignment(
    this.assignment,
    this.stops,
    this.startAddress,
    this.clientMap,
    this.shipmentMap,
    this.geocodeAddressMap,
    { this.tasks, this.extra }
  );

  final Assignment assignment;
  final List<DecorStop> stops;

  final AssignmentExtra extra;

  @JsonKey(name: 'start_address')
  final Address startAddress;

  @JsonKey(name: 'client_map')
  final Map<String, Client> clientMap;

  @JsonKey(name: 'shipment_map')
  final Map<String, Shipment> shipmentMap;

  @JsonKey(name: 'geocode_address_map')
  final Map<String, GeocodeAddress> geocodeAddressMap;

  List<Task> tasks;

  get taskCount {
    Map<TaskType, int> count = {};
    for (Task d in tasks) {
      if (count.containsKey(d.type)) {
        count[d.type] = count[d.type] + 1;
      } else {
        count[d.type] = 1;
      }
    }
    return count;
  }

  DecorAssignment withAssignment(Assignment assignment) {
    return DecorAssignment(
      assignment,
      this.stops,
      this.startAddress,
      this.clientMap,
      this.shipmentMap,
      this.geocodeAddressMap,
      tasks: tasks,
      extra: extra,
    )..metadata = this.metadata;
  }

  DecorAssignment withTasks(List<Task> tasks) {
    return DecorAssignment(
      this.assignment,
      this.stops,
      this.startAddress,
      this.clientMap,
      this.shipmentMap,
      this.geocodeAddressMap,
      tasks: tasks,
      extra: extra,
    )..metadata = this.metadata;
  }

  DecorAssignment clone({Assignment assignment, List<DecorStop> stops, List<Task> tasks, AssignmentExtra extra, Map<String, dynamic> metadata, Map<String, GeocodeAddress> geocodeAddressMap }) {
    return DecorAssignment(
      assignment ?? this.assignment,
      stops ?? this.stops,
      this.startAddress,
      this.clientMap,
      this.shipmentMap,
      geocodeAddressMap ?? this.geocodeAddressMap,
      tasks: tasks ?? this.tasks,
      extra: extra ?? this.extra,
    )..metadata = metadata ?? this.metadata;
  }

  Task get currentTask {
    if (tasks == null || tasks.isEmpty) return null;
    List<Task> openTasks = tasks.skipWhile((t) => t.isCompleted).toList();
    return openTasks.isEmpty ? null : openTasks[0];
  }

  factory DecorAssignment.fromJson(Map<String, dynamic> json) => _$DecorAssignmentFromJson(json);
  Map<String, dynamic> toJson() => _$DecorAssignmentToJson(this);
}