import 'package:axlehire_flutter_model/src/event_location.dart';
import 'package:axlehire_flutter_model/src/mixin/attributes.dart';
import 'package:axlehire_flutter_model/src/reference.dart';
import 'package:json_annotation/json_annotation.dart';
part 'event.g.dart';

@JsonSerializable()

class Event with AttributesMixin {
  Event(
    this.id,
    this.category,
    this.type,
    this.action,
    {this.fact, this.evidence, this.state, this.origin, this.ts, this.location, this.source, this.object, this.subject, this.ref, this.rel, this.ephemeral}
  );

  final String id;
  final String category;
  final String     type;
  final String     action;
  final String     origin;   // null for self
  final int       ts;       // when
  final EventLocation   location; // physical location
  final Reference  source;   // where. Can be an application / worker / another event
  final Reference  object;   // whom, main target of the event
  final Reference  subject;  // who, for ex. driver / dispatcher
  final Reference  ref;      // related entity
  final Reference  rel;      // another related entity
  final bool ephemeral;      // no side effect

  Map<String, String> fact = {};     // state before action, for ex. status
  Map<String, String> evidence = {}; // supporting evidence of the action, for example driver remark
  Map<String, String> state = {};    // updated state after action

  bool dirty = true;

  void clear() {
    this.dirty = false;
  }

  // convinient function to update event
  void withState(String name, String value) {
    this.state[name] = value;
  }
  void withStates(Map<String, String> state) {
    if (this.state == null)
      this.state = Map.from(state);
    else
      this.state.addAll(state);
  }
  void withFact(String name, String value) {
    this.fact[name] = value;
  }
  void withFacts(Map<String, String> fact) {
    if (this.fact == null)
      this.fact = Map.from(fact);
    else
      this.fact.addAll(fact);
  }
  void withEvidence(String name, String value) {
    this.evidence[name] = value;
  }
  void withEvidences(Map<String, String> evidence) {
    if (this.evidence == null)
      this.evidence = Map.from(evidence);
    else
      this.evidence.addAll(evidence);
  }

  factory Event.fromJson(Map<String, dynamic> json) => _$EventFromJson(json);
  Map<String, dynamic> toJson() => _$EventToJson(this);
}