import 'dart:io';

import 'package:axlehire_flutter_model/src/messenger_activity_log.dart';
import 'package:axlehire_flutter_model/src/messenger_reaction.dart';
import 'package:json_annotation/json_annotation.dart';
import 'dart:convert' as convert;

part 'messenger.g.dart';

@JsonSerializable()
class Message {
  String ts;
  String body;
  String id;
  @JsonKey(name: "topic_id")
  String topicId;
  @JsonKey(name: "is_sync")
  int isSync;
  List<Reaction> reactions;
  @JsonKey(name: "activity_logs")
  List<ActivityLog> activityLogs;
  @JsonKey(name: "ref_opinion_map")
  Map<String, dynamic> refOpinionMaps;
  @JsonKey(name: "ref_opinion_expired_ts")
  String refOpinionExpiredTs;

  String sender;
  bool isSeen;
  String path;
  File file;

  Message({this.ts, this.body, this.id, this.topicId, this.isSync,
    this.reactions, this.activityLogs, this.refOpinionMaps,
    this.refOpinionExpiredTs, this.sender, this.path, this.isSeen = true, this.file});

  factory Message.fromJson(Map<String, dynamic> json, {bool isConvertedObjectFromString = false}) {
    if (isConvertedObjectFromString) {
      var map = Map<String,dynamic>.from(json);
      map['reactions'] = convert.jsonDecode(json['reactions']);
      map['activity_logs'] = convert.jsonDecode(json['activity_logs']);
      map['ref_opinion_map'] = convert.jsonDecode(json['ref_opinion_map']);
      return _$MessageFromJson(map);
    }
    return _$MessageFromJson(json);
  }

  Map<String, dynamic> toJson({bool isConvertedObjectToString = false}) {
    if (isConvertedObjectToString) {
      var map = _$MessageToJson(this);
      map['reactions'] = convert.jsonEncode(map['reactions']);
      map['activity_logs'] = convert.jsonEncode(map['activity_logs']);
      map['ref_opinion_map'] = convert.jsonEncode(map['ref_opinion_map']);
      return map;
    }
    return _$MessageToJson(this);
  }

  bool get isViewed {
    if (this.activityLogs != null && this.activityLogs.length > 0) {
      for (int i = 0; i < this.activityLogs.length; i++) {
        if (this.activityLogs[i].activity == "VIEW") {
          return true;
        }
      }
    }
    return false;
  }

  Message markAsView() {
    var newActivityLog = ActivityLog(activity: "VIEW", ts: DateTime.now().toUtc().toIso8601String());
    this.activityLogs?.add(newActivityLog);
    return this;
  }

  Message reactToMessage(Reaction reaction) {
    this.reactions.add(reaction);
    return this;
  }

  Message copyWith({String ts, String body, String id, String topicId, int isSync,
    List<Reaction> reactions, List<ActivityLog> activityLogs, Map<String, dynamic> refOpinionMaps,
    String refOpinionExpiredTs}) {
    return Message(
      ts: ts ?? this.ts,
      body: body ?? this.body,
      id: id ?? this.id,
      topicId: topicId ?? this.topicId,
      isSync: isSync ?? this.isSync,
      reactions: reactions ?? this.reactions,
      activityLogs: activityLogs ?? this.activityLogs,
      refOpinionMaps: refOpinionMaps ?? this.refOpinionMaps,
      refOpinionExpiredTs: refOpinionExpiredTs ?? this.refOpinionExpiredTs
    );
  }
}
