class MetaDataMixin {
  Map<String, dynamic> metadata;

  void setMeta(String key, dynamic value) {
    if (this.metadata == null) {
      this.metadata = new Map();
    }
    this.metadata[key] = value;
  }

  void setMetas(Map<String, dynamic> vals) {
    if (this.metadata == null) {
      this.metadata = new Map();
    }
    this.metadata.addAll(vals);
  }

  String getMeta(String key, { String defaultValue = null }) {
    if (this.metadata != null && this.metadata.containsKey(key)) {
      dynamic val = this.metadata[key];
      return val.toString();
    }
    return defaultValue;
  }

  bool getMetaBool(String key, { bool defaultValue = true }) {
    if (this.metadata != null && this.metadata.containsKey(key)) {
      dynamic val = this.metadata[key];
      if (val is bool) {
        return val;
      } else if (val is String) {
        return val.toLowerCase() == 'true';
      } else {
        throw new FormatException('metadata is not of bool-compatible type');
      }
    }
    return defaultValue;
  }

  int getMetaInt(String key, { int defaultValue = 0 }) {
    if (this.metadata != null && this.metadata.containsKey(key)) {
      dynamic val = this.metadata[key];
      if (val is int) {
        return val;
      } else if (val is String) {
        return int.parse(val);
      } else {
        throw new FormatException('metadata is not of int-compatible type');
      }
    }
    return defaultValue;
  }

  double getMetaDbl(String key, { double defaultValue = 0.0 }) {
    if (this.metadata != null && this.metadata.containsKey(key)) {
      dynamic val = this.metadata[key];
      if (val is double) {
        return val;
      } else if (val is String) {
        return double.parse(val);
      } else {
        throw new FormatException('metadata is not of int-compatible type');
      }
    }
    return defaultValue;
  }

}