import 'package:axlehire_flutter_model/src/paystub_breakdown.dart';
import 'package:axlehire_flutter_model/src/paystub_breakdown_type.dart';
import 'package:axlehire_flutter_model/src/payroll_date_transaction.dart';
import 'package:json_annotation/json_annotation.dart';
part 'paystub.g.dart';

@JsonSerializable()

class PayStub {
  @JsonKey(name: "id")
  String id;
  @JsonKey(name: "start_ts")
  String startTs;
  @JsonKey(name: "end_ts")
  String endTs;
  @JsonKey(name: "transaction_count")
  int transactionCount;
  @JsonKey(name: "amount")
  double amount;
  @JsonKey(name: "breakdown")
  Breakdown breakdown;
  @JsonKey(name: "transactions_by_date")
  List<DateTransaction> dateTransactions;

  PayStub(this.id, this.startTs, this.endTs, this.transactionCount, this.amount,
      this.breakdown, this.dateTransactions);

  factory PayStub.fromJson(Map<String, dynamic> json) => _$PayStubFromJson(json);
  Map<String, dynamic> toJson() => _$PayStubToJson(this);

  String taskAndSummaryCount() {
    if (breakdown.breakdownsByType.length == 0) return "Empty";
    int taskCount = 0;
    int tipCount = 0;
    for (BreakdownType breakdownType in breakdown.breakdownsByType) {
      switch(breakdownType.type) {
        case "OUTVOICE - ROUTE_DRIVING":
          taskCount += breakdownType.transactionCount;
          break;
        case "OUTVOICE - TIP":
          tipCount += breakdownType.transactionCount;
          break;
      }
    }

    String tipCountString = tipCount > 0 ? "$tipCount Tips" : "";
    String taskCountString = taskCount > 0 ? "$taskCount Tasks" : "";
    return "$taskCountString ${taskCountString.isNotEmpty ? "&" : ""} $tipCountString";
  }

  String get status => this.breakdown.breakdownsByStatus[0]?.status;
}
