import 'package:axlehire_flutter_model/src/parcel.dart';
import 'package:json_annotation/json_annotation.dart';
import './address.dart';
import './customer.dart';

part 'shipment.g.dart';


/// An annotation for the code generator to know that this class needs the
/// JSON serialization logic to be generated.
@JsonSerializable()

class Shipment {
  Shipment(
    this.id,
    this.clientId,
    this.assignmentId,
    this.internalId,
    this.customer,
    this.deliveryProofPhotoRequired,
    this.signatureRequired,
    this.smsEnabled,
    this.idRequired,
    this.timezone,
    this.trackingCode,
    this.tags,
    this.pickupAddress,
    this.dropoffAddress,
    this.earliestDropoffTs,
    this.latestDropoffTs,
    this.earliestPickupTs,
    this.latestPickupTs,
    this.inboundStatus,
    this.status,
    this.pickupNote,
    this.dropoffNote,
    this.rdi,
    this.dropoffAccessCode,
    this.dropoffAdditionalInstruction,
    this.dropoffNavigationDifficulty,
    this.dropoffUncharted,
    this.deliveryItems,
    this.pickupGeocoderId,
    this.dropoffGeocoderId,
    { this.parcels, this.items, this.serviceLevel, this.serviceType }
  );

  final int id;

  @JsonKey(name: 'client_id')
  final int clientId;

  @JsonKey(name: 'assignment_id')
  final int assignmentId;

  @JsonKey(name: 'internal_id')
  final String internalId;

  final Customer customer;

  @JsonKey(name: "delivery_proof_photo_required")
  final bool deliveryProofPhotoRequired;

  @JsonKey(name: "sms_enabled")
  final bool smsEnabled;

  @JsonKey(name: "signature_required")
  final bool signatureRequired;

  @JsonKey(name: "id_required")
  final bool idRequired;

  final String timezone;

  @JsonKey(name: 'tracking_code')
  final String trackingCode;

  final List<String> tags;

  @JsonKey(name: 'pickup_address')
  final Address pickupAddress;

  @JsonKey(name: 'dropoff_address')
  final Address dropoffAddress;

  @JsonKey(name: 'dropoff_earliest_ts')
  final DateTime earliestDropoffTs;

  @JsonKey(name: 'dropoff_latest_ts')
  final DateTime latestDropoffTs;

  @JsonKey(name: 'pickup_earliest_ts')
  final DateTime earliestPickupTs;

  @JsonKey(name: 'pickup_latest_ts')
  final DateTime latestPickupTs;

  @JsonKey(name: 'inbound_status')
  final String inboundStatus;

  final String status;

  @JsonKey(name: 'pickup_note')
  final String pickupNote;

  @JsonKey(name: 'dropoff_note')
  final String dropoffNote;

  final String rdi;

  @JsonKey(name: "dropoff_access_code")
  final String dropoffAccessCode;

  @JsonKey(name: "dropoff_additional_instruction")
  final String dropoffAdditionalInstruction;

  @JsonKey(name: "dropoff_navigation_difficulty")
  final String dropoffNavigationDifficulty;

  @JsonKey(name: "dropoff_uncharted")
  final bool dropoffUncharted;

  @JsonKey(name: 'delivery_items')
  final String deliveryItems;

  @JsonKey(name: 'pickup_geocoder_id')
  final String pickupGeocoderId;

  @JsonKey(name: 'dropoff_geocoder_id')
  final String dropoffGeocoderId;

  @JsonKey(name: "service_level") final String serviceLevel;
  @JsonKey(name: "service_type")  final String serviceType;

  List<Parcel> parcels;
  List<Parcel> items;

  factory Shipment.fromJson(Map<String, dynamic> json) => _$ShipmentFromJson(json);
  Map<String, dynamic> toJson() => _$ShipmentToJson(this);
}