import 'package:json_annotation/json_annotation.dart';

import 'constant/stop_status.dart';
part 'stop.g.dart';

@JsonSerializable()

class Stop {
  Stop(
    this.id,
    this.type,
    this.shipmentId,
    this.sequenceId,
    this.assignmentId,
    this.clientId,
    this.correspondingStopId,
    this.predictedArrivalTs,
    this.predictedDepartureTs,
    this.actualArrivalTs,
    this.actualDepartureTs,
    this.status,
    this.reason,
    this.remark,
    this.dispatcherNote,
    this.uid,
    this.travelDistanceFromPreviousStop,
    this.travelTimeFromPreviousStop
  );

  final int id;
  final String type;

  @JsonKey(name: 'shipment_id')
  final int shipmentId;

  @JsonKey(name: 'sequence_id')
  final int sequenceId;

  @JsonKey(name: 'assignment_id')
  final int assignmentId;

  @JsonKey(name: 'client_id')
  final int clientId;

  @JsonKey(name: 'corresponding_stop_id')
  final int correspondingStopId;

  // time
  @JsonKey(name: 'predicted_arrival_ts')
  final DateTime predictedArrivalTs;

  @JsonKey(name: 'predicted_departure_ts')
  final DateTime predictedDepartureTs;

  @JsonKey(name: 'actual_arrival_ts')
  final DateTime actualArrivalTs;

  @JsonKey(name: 'actual_departure_ts')
  final DateTime actualDepartureTs;

  final String status;
  final String reason;
  final String remark;
  final String uid;

  @JsonKey(name: 'travel_distance_from_previous_stop')
  final double travelDistanceFromPreviousStop;

  @JsonKey(name: 'travel_time_from_previous_stop')
  final double travelTimeFromPreviousStop;

  @JsonKey(name: 'dispatcher_note')
  final String dispatcherNote;

  @override
  String toString() {
    return 'STOP::$id';
  }

  bool get isStarted {
    return status != null && StopStatus.LIVE_STATUSES.contains(status);
  }

  bool get isCompleted {
    return status != null && StopStatus.TERMINAL_STATUSES.contains(status);
  }

  factory Stop.fromJson(Map<String, dynamic> json) => _$StopFromJson(json);
  Map<String, dynamic> toJson() => _$StopToJson(this);

  /// convenient function
  Stop clone({DateTime actualArrivalTs, DateTime actualDepartureTs, String status, String reason, String remark}) {
    return Stop(
      this.id,
      this.type,
      this.shipmentId,
      this.sequenceId,
      this.assignmentId,
      this.clientId,
      this.correspondingStopId,
      this.predictedArrivalTs,
      this.predictedDepartureTs,
      actualArrivalTs ?? this.actualArrivalTs,
      actualDepartureTs ?? this.actualDepartureTs,
      status ?? this.status,
      reason ?? this.reason,
      remark ?? this.remark,
      this.dispatcherNote,
      this.uid,
      this.travelDistanceFromPreviousStop,
      this.travelTimeFromPreviousStop,
    );
  }

  bool get isFailed => status == StopStatus.FAILED;
  bool get isSucceeded => status == StopStatus.SUCCEEDED;
  bool get isReady => status == StopStatus.READY;
  bool get isEnRoute => status == StopStatus.EN_ROUTE;
}
