import 'package:json_annotation/json_annotation.dart';

import 'task.dart';

part 'sub_task.g.dart';

@JsonSerializable()

class SubTask {
  SubTask(this.name, { this.completed = false });

  final String name;
  final bool completed;

  static const String TRANSIT = 'TRANSIT';
  static const String CHECK_ID = 'CHECK_ID';
  static const String PERFORM = 'PERFORM';
  static const String PREVIEW = 'PREVIEW';
  static const String NEXT = 'NEXT';

  SubTask complete() {
    return SubTask(this.name, completed: true);
  }

  SubTask undo() {
    return SubTask(this.name, completed: false);
  }

  @override
  toString() {
    return '$name: $completed';
  }

  factory SubTask.fromJson(Map<String, dynamic> json) => _$SubTaskFromJson(json);
  Map<String, dynamic> toJson() => _$SubTaskToJson(this);
}

class TransitTo extends SubTask {
  TransitTo({bool completed = false}): super(SubTask.TRANSIT, completed: completed);
}

class CheckId extends SubTask {
  CheckId(this.key, this.value, {bool completed = false}): super(SubTask.CHECK_ID, completed: completed);
  final String key;
  final String value;
}

class Perform extends SubTask {
  Perform({bool completed = false}): super(SubTask.PERFORM, completed: completed);
}
class Next extends SubTask {
  Next({this.task, bool completed = false}): super(SubTask.NEXT, completed: completed);
  final Task task;
}

class Preview extends SubTask {
  Preview({bool completed = false}): super(SubTask.PREVIEW, completed: completed);
}
