import 'package:axlehire_flutter_model/axlehire_model.dart';
import 'package:axlehire_flutter_model/src/sub_task.dart';

import 'address.dart';
import 'task_type.dart';
import 'decor_stop.dart';
import 'package:json_annotation/json_annotation.dart';

part 'task.g.dart';

@JsonSerializable()

class Task {
  Task(this.sequenceId, this.type, { this.address, this.geocodeAddress, this.instruction, this.stops });

  @JsonKey(name: 'sequence_id')
  final int sequenceId;
  final TaskType type; // PICKUP - DROPOFF - RETURN
  final Address address;
  final GeocodeAddress geocodeAddress;
  final String instruction;
  Assignment assignment;
  List<DecorStop> stops;
  List<SubTask> subs;

  static const String META_KEY_EARLIEST_START = 'earliest_start_lead_time';

  SubTask get currentSubTask {
    return subs.firstWhere((s) => !s.completed, orElse: null);
  }

  bool get isCompleted {
    return subs == null || subs.every((s) => s.completed);
  }

  bool get isDone {
    return stops == null || stops.every((s) => s.stop.isCompleted);
  }

  bool get isStarted {
    return stops == null || stops.every((s) => s.stop.isStarted);
  }

  DateTime get earliestCanStartTime {
    if (stops == null || stops.isEmpty) return null;
    List<DateTime> limit = stops.where((s) => s.getMeta(META_KEY_EARLIEST_START) != null)
      .map((s) => Pair(s.stop.type == StopType.DROP_OFF ? s.shipment.earliestDropoffTs : s.shipment.earliestPickupTs, s.getMetaInt(META_KEY_EARLIEST_START, defaultValue: 120)))
      .where((p) => p.left != null)
      .map((p) => p.left.add(Duration(minutes: -p.right)))
      .toList();
    if (limit.isEmpty) return null;
    
    limit.sort((a,b) => a.compareTo(b));
    return limit.last;
  }

  @override
  toString() {
    return '${type.type} [$address] [$currentSubTask]';
  }

  factory Task.fromJson(Map<String, dynamic> json) => _$TaskFromJson(json);
  Map<String, dynamic> toJson() => _$TaskToJson(this);
}
